	.text
	.globl chmod_parser
	.globl chmod_parser_is_dir
// <- %eax = original file mode
//    %ebx = start of zero-terminated character mode
//    %ebp = umask
// -> %eax = new file mode
//    eax=-1 = parser error

// in the parser:
// %ebx = address in the string
// %edx = operation ( 0 - '=', 1 - '+', 2 - '-')
// %ecx = temporal register
// %esi = wholist

//%start             symbolic_mode
//%%
//symbolic_mode    : section
//                 | symbolic_mode ',' section
//                 ;
//section          : actionlist
//                 | wholist actionlist
//                 ;
//wholist          : who
//                 | wholist who
//                 ;
//who              : 'u' | 'g' | 'o' | 'a'
//                 ;
//actionlist       : action
//                 | actionlist action
//                 ;
//action           : op
//                 | op permlist
//                 | op permcopy
//                 ;
//permcopy         : 'u' | 'g' | 'o'
//                 ;
//op               : '+' | '-' | '='
//                 ;
//permlist         : perm
//                 | perm permlist
//                 ;
//perm             : 'r' | 'w' | 'x' | 'X' | 's'
//                 ;

chmod_parser:
	pusha
	movl	%eax,old_mode
	decl	%ebx
	movl	$state_init,state_machine
start_of_parser:
	incl	%ebx
	movb	(%ebx),%al
	testb	%al,%al
	jz	exit_parser
rec_start_of_parser:
// The parser is a state machine
	jmp	*state_machine
// INITIALIZATION STATE
state_init:
// reset the chmod_parser_modechange
	xorl	%esi,%esi
// check the next symbol (TODO maybe jumptable would be smaller)
	cmpb	$'u',%al
	jne	init_not_u
	movl	$state_wholist,state_machine
	jmp	rec_start_of_parser
init_not_u:
	cmpb	$'g',%al
	jne	init_not_g
	movl	$state_wholist,state_machine
	jmp	rec_start_of_parser
init_not_g:
	cmpb	$'o',%al
	jne	init_not_o
	movl	$state_wholist,state_machine
	jmp	rec_start_of_parser
init_not_o:
	cmpb	$'a',%al
	jne	init_not_a
	movl	$state_wholist,state_machine
	jmp	rec_start_of_parser
init_not_a:
	cmpb	$'=',%al
	jne	init_not_equals
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
init_not_equals:
	cmpb	$'+',%al
	jne	init_not_plus
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
init_not_plus:
	cmpb	$'-',%al
	jne	init_not_minus
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
init_not_minus:
	popa	
	movl	$-1,%eax
	ret
// PARSING THE WHOLIST (u|g|o|a|)
// add the corresponding bits to the chmod_parser_modechange's whomask
state_wholist:
	cmpb	$'u',%al
	jne	wholist_not_u
	orl	$0700,%esi
	jmp	start_of_parser
wholist_not_u:
	cmpb	$'g',%al
	jne	wholist_not_g
	orl	$0070,%esi
	jmp	start_of_parser
wholist_not_g:
	cmpb	$'o',%al
	jne	wholist_not_o
	orl	$0007,%esi
	jmp	start_of_parser
wholist_not_o:
	cmpb	$'a',%al
	jne	wholist_not_a
	orl	$0777,%esi
	jmp	start_of_parser
wholist_not_a:
	cmpb	$'=',%al
	jne	wholist_not_equals
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
wholist_not_equals:
	cmpb	$'+',%al
	jne	wholist_not_plus
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
wholist_not_plus:
	cmpb	$'-',%al
	jne	wholist_not_minus
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
wholist_not_minus:
	popa
	movl	$-1,%eax
	ret

// PARSING THE ACTION (+ | = | -)
// this is the only obligatory part
state_action:
	testl	%esi,%esi
	jnz	non_zero_wholist
	movl	%ebp,%esi
non_zero_wholist:

	cmpb	$'=',%al
	jne	action_not_equals
	movb	$0,%dl
	movl	%esi,%eax
	notl	%eax
	andl	%eax,old_mode
	movl	$state_permlist,state_machine
	jmp	start_of_parser
action_not_equals:
	cmpb	$'+',%al
	jne	action_not_plus
	movb	$1,%dl
	movl	$state_permlist,state_machine
	jmp	start_of_parser
action_not_plus:
	cmpb	$'-',%al
	jne	action_not_minus
	movb	$2,%dl
	movl	$state_permlist,state_machine
	jmp	start_of_parser
action_not_minus:
	popa
	movl	$-1,%eax
	ret

// PARSING THE PERMLIST (rwxXstugo)
// we come here right after parsing =/+/-
// here all the actual mode changes are performed
state_permlist:
	cmpb	$'r',%al
	jne	permlist_not_r
	movl	%esi,%eax
	andl	$0444,%eax
	cmpb	$2,%dl
	je	permlist_r_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_r_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_r:
	cmpb	$'w',%al
	jne	permlist_not_w
	movl	%esi,%eax
	andl	$0222,%eax
	cmpb	$2,%dl
	je	permlist_w_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_w_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_w:
	cmpb	$'x',%al
	jne	permlist_not_x
	movl	%esi,%eax
	andl	$0111,%eax
	cmpb	$2,%dl
	je	permlist_x_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_x_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_x:
	cmpb	$'t',%al
	jne	permlist_not_t
	movl	$01000,%eax
	cmpb	$2,%dl
	je	permlist_t_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_t_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_t:
	cmpb	$'u',%al
	jne	permlist_not_u
	movl	old_mode,%eax
	andl	$0700,%eax
	movl	%eax,%ecx
	shrl	$3,%ecx
	orl	%eax,%ecx
	shrl	$3,%ecx
	orl	%eax,%ecx
	movl	%esi,%eax
	andl	%ecx,%eax
	cmpb	$2,%dl
	je	permlist_u_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_u_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_u:
	cmpb	$'g',%al
	jne	permlist_not_g
	movl	old_mode,%eax
	andl	$070,%eax
	movl	%eax,%ecx
	shrl	$3,%ecx
	orl	%eax,%ecx
	shll	$3,%eax
	orl	%eax,%ecx
	movl	%esi,%eax
	andl	%ecx,%eax
	cmpb	$2,%dl
	je	permlist_g_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_g_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_g:
	cmpb	$'o',%al
	jne	permlist_not_o
	movl	old_mode,%eax
	andl	$007,%eax
	movl	%eax,%ecx
	shll	$3,%eax
	orl	%eax,%ecx
	shll	$3,%eax
	orl	%eax,%ecx
	movl	%esi,%eax
	andl	%ecx,%eax
	cmpb	$2,%dl
	je	permlist_o_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_o_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_o:
	cmpb	$'s',%al
	jne	permlist_not_s
	xorl	%ecx,%ecx
	movl	%esi,%eax
	andl	$0070,%eax
	jz	permlist_not_setgid
	orl	$02000,%ecx
permlist_not_setgid:
	movl	%esi,%eax
	andl	$0700,%eax
	jz	permlist_not_setuid
	orl	$04000,%ecx
permlist_not_setuid:
	testl	%ecx,%ecx
	jz	parser_error
	cmpb	$2,%dl
	je	permlist_s_sub
	orl	%ecx,old_mode
	jmp	start_of_parser
permlist_s_sub:
	notl	%ecx
	andl	%ecx,old_mode
	jmp	start_of_parser

permlist_not_s:
	cmpb	$'X',%al
	jne	permlist_not_X
	testl	$0111,old_mode
	jz	start_of_parser
	cmpb	$1,chmod_parser_is_dir
	jne	start_of_parser
	movl	%esi,%eax
	andl	$0111,%eax
	cmpb	$2,%dl
	je	permlist_X_sub
	orl	%eax,old_mode
	jmp	start_of_parser
permlist_X_sub:
	notl	%eax
	andl	%eax,old_mode
	jmp	start_of_parser

permlist_not_X:
	cmpb	$'=',%al
	jne	permlist_not_equals
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
permlist_not_equals:
	cmpb	$'+',%al
	jne	permlist_not_plus
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
permlist_not_plus:
	cmpb	$'-',%al
	jne	permlist_not_minus
	movl	$state_action,state_machine
	jmp	rec_start_of_parser
permlist_not_minus:
	cmpb	$',',%al
	jne	permlist_not_comma
	movl	$state_init,state_machine
	jmp	start_of_parser
permlist_not_comma:

parser_error:
	popa
	movl	$-1,%eax
	ret

exit_parser:
	popa
	movl	old_mode,%eax
	ret

	.data
state_machine:
	.long	state_init
// MUST be set to 1 by external program
chmod_parser_is_dir:
	.byte	0
	.bss
// TODO: move all into registers
	.lcomm	old_mode,4

