// ASMIX Commands and Utilities
// (c) Sergei Zubkov (Cubbi) 1999
//
// dirname.s - return the directory portion of pathname
//
// $Id: dirname.s,v 1.1.1.1 1999/10/04 20:54:17 cubbi Exp $
DIRNAME_V_MAJ = 1
DIRNAME_V_MIN = 0

.include "config.i"
	.globl _start
_start:
	cld
	popl	%ecx
// this was argc - 1 means we are called without arguments
	decl	%ecx
	jz	error_exit
	popl	%edi
// this is argv
arguments_cycle:
// skip argv[0]
	pushl	%ecx
	movl	$-1,%ecx
	xorl	%eax,%eax
	repne
	scasb
	popl	%ecx
// Checks for arguments
.if OPTIONS
	cmpb	$1,end_of_options
	je	skip_options
// Check for --
	cmpw	$0x2D2D,(%edi)
	jne	not_dashdash
	movb	$1,end_of_options
	loop	arguments_cycle
not_dashdash:
// Check for -options
.if FEATURE_VERSION
// Check for -V
	cmpw	$0x562D,(%edi)
	je	show_version
.endif
// exit with error message about unrecognized option
// and with the reminder about --
// TBD (if +errors)
// not an option -> options no more
	movb	$1,end_of_options
.endif

.if OPTIONS
//	loop	arguments_cycle
skip_options:
	jcxz	error_exit
.endif


// the dirname itself starts here

// if we got empty string from the very start - change it into "."
	call	strlen
	testl	%ecx,%ecx
	jne	not_empty_string_start
	movw	$0x002E,(%edi)
	jmp	end_of_parse
not_empty_string_start:

// remove all trailing slashes
	call	remove_trailing

// if we got empty string - change it into "/"
	call	strlen
	testl	%ecx,%ecx
	jne	not_empty_string
	movw	$0x002F,(%edi)
	jmp	end_of_parse
not_empty_string:

// check if what's left has no slashes in it, change it into "." then
	call	strlen
	pushl	%ecx
	pushl	%edi
	incl	%ecx
	movb	$0x2F,%al
	repne
	scasb
	popl	%edi
	testl	%ecx,%ecx
	jne	some_slashes_left
	movw	$0x002E,(%edi)
	jmp	end_of_parse
some_slashes_left:
	popl	%ecx

// remove all non-slash trailing characters
	pushl	%edi
	call	strlen
	addl	%ecx,%edi
	decl	%edi
remove_trailing_letters:
	cmpb	$0x2F,(%edi)
	je	no_more_trailing_letters
	movb	$0,(%edi)
	decl	%edi
	jmp	remove_trailing_letters
no_more_trailing_letters:
	popl	%edi

// remove trailing slashes again
	call	remove_trailing

// if we got empty string again - change it into "/"
	call	strlen
	testl	%ecx,%ecx
	jne	not_empty_string_again
	movw	$0x002F,(%edi)
	jmp	end_of_parse
not_empty_string_again:

end_of_parse:
// add trailing \n
	call	strlen
	movb	$012,(%edi,%ecx,1)
	incl	%ecx

.if SYSCALL_unix
        pushl   %ecx
        pushl   %edi
        pushl   $1
        movl    $SYSCALL_WRITE,%eax
        pushl   %eax
        .byte   0x9A
        .long   0
        .word   7
        addl    $16,%esp
.endif
.if SYSCALL_linux
        movl    %ecx,%edx
        movl    %edi,%ecx
        movl    $1,%ebx
        movl    $SYSCALL_WRITE,%eax
        int     $0x80
.endif

// exit the program - exit(0)
exit:
	xorl	%ebx,%ebx
.if SYSCALL_unix
	pushl	%eax
	movl	$SYSCALL_EXIT,%eax
	pushl	%eax
        .byte   0x9A
        .long   0
        .word   7
.endif
.if SYSCALL_linux
	movl	$SYSCALL_EXIT,%eax
	int	$0x80
.endif

error_exit:
	xorl	%ebx,%ebx
	incl	%ebx
	movl	$SYSCALL_EXIT,%eax
.if SYSCALL_unix
	pushl	%ebx
	pushl	%eax
        .byte   0x9A
        .long   0
        .word   7
.endif
.if SYSCALL_linux
	int	$0x80
.endif


.if FEATURE_VERSION
show_version:
.if SYSCALL_unix
        pushl   $version_l
        pushl   $version
        pushl   $1
        movl    $SYSCALL_WRITE,%eax
        pushl   %eax
        .byte   0x9A
        .long   0
        .word   7
        addl    $16,%esp
.endif
.if SYSCALL_linux
	movl	$version_l,%edx
	movl	$version,%ecx
	movl	$1,%ebx
	movl	$SYSCALL_WRITE,%eax
	int	$0x80
.endif
	jmp	exit
.endif

	.data
.if OPTIONS
end_of_options:
	.byte	0
.endif
.if FEATURE_VERSION
version:
	.ascii	"dirname.s "
	.byte	'0'+DIRNAME_V_MAJ
	.ascii	"."
	.byte	'0'+DIRNAME_V_MIN
	.ascii	"+version"
	.ascii	" ASMIX CU "
	.byte	'0'+ASMIX_VER_MAJ
	.ascii	"."
	.byte	'0'+ASMIX_VER_MIN
	.ascii	"."
	.byte	'0'+ASMIX_VER_REV
	.ascii	"\012"
version_l = .-version
.endif


	.text
// In: %edi = start of zero-terminated string
// Out: %ecx = length of the string
// Modifies: %eax
// Requires: cld
strlen:
	pushl	%edi
	movl	$-1,%ecx
	xorl	%eax,%eax
	repne
	scasb
	movl	%edi,%ecx
	subl	(%esp),%ecx
	decl	%ecx
	popl	%edi
	ret

// In: %edi = string
// Out: string is modified to have all trailing slashes replaced with zeroes
remove_trailing:
	pushl	%edi
	call	strlen
	addl	%ecx,%edi
	decl	%edi
rem_trailing:
	cmpb	$0x2F,(%edi)	
	jne	no_more_trailing
	movb	$0,(%edi)
	decl	%edi
	jmp	rem_trailing
no_more_trailing:
	popl	%edi
	ret
